import { EntityRawSchemaArray } from "./Entities";

/**
 * Type used to check if a schema is loosely typed.
 * Use this to create conditional types.
 *
 * In the given EntityRawSchemaArray, if the entities type is a string, the schema is loosely typed.
 * If the schema is strongly typed, the entities type would be a word (which is stricter than string).
 */
export type IsLooseSchema<E extends EntityRawSchemaArray> =
  string extends E[number]["type"] ? true : false;

/**
 * Checks if the given entity type is present in the EntityRawSchemaArray.
 * Use this to create conditional types.
 */
export type IsETypeInStrictSchema<
  E extends EntityRawSchemaArray,
  EType extends string
> = EType extends E[number]["type"] ? true : false;

/**
 * Type representing the entity's raw schema matching this type.
 * This type only works for EntityRawSchemaArray with literals as "type" property.
 */
export type EntityRawSchemaByType<
  E extends EntityRawSchemaArray,
  T extends string
> = E[number] extends infer ESchema
  ? ESchema extends { type: T }
    ? ESchema
    : never
  : never; // Will never happen E[number] always has a type
