import {
  EntityRawSchemaArray,
  RelationDefinitionRawSchemaArray,
  Client,
  Schema,
  getLooseSchema,
} from "@cubicweb/client";
import { CreateParams, CreateResult, RaRecord } from "./types";
import { makeRecursiveCreateRequest } from "./utils/createParams";

export const getCreate = <
  E extends EntityRawSchemaArray,
  R extends RelationDefinitionRawSchemaArray<E>
>(
  client: Client,
  schema: Schema<E, R>
) => {
  const create = async <RecordType extends RaRecord>(
    resource: string,
    { data }: CreateParams
  ): Promise<CreateResult<RecordType>> => {
    const newData = await client.transaction(async (transaction) => {
      return (await makeRecursiveCreateRequest(
        getLooseSchema(schema),
        resource,
        data,
        transaction
      )) as RecordType;
    });
    return { data: newData } as CreateResult<RecordType>;
  };
  return { create };
};
