import { EntityRawSchemaArray, Client } from "@cubicweb/client";
import {
  DeleteManyParams,
  DeleteManyResult,
  DeleteParams,
  DeleteResult,
  RaRecord,
} from "./types";
import { executeApiRequest } from "./utils/requests";

export const getDelete = <E extends EntityRawSchemaArray>(client: Client) => {
  const _delete = async <RecordType extends RaRecord>(
    resource: E[number]["type"],
    data: DeleteParams
  ): Promise<DeleteResult<RecordType>> => {
    const requestStr = `DELETE ${resource} X WHERE X eid %(eid)s`;

    return executeApiRequest(client, requestStr, { eid: data.id }).then(() => {
      return { data: data.previousData as unknown as RecordType };
    });
  };

  const deleteMany = async (
    resource: E[number]["type"],
    data: DeleteManyParams
  ): Promise<DeleteManyResult> => {
    const requestStr = `DELETE ${resource} X WHERE X eid IN (${data.ids.join(
      ", "
    )})`;

    return executeApiRequest(client, requestStr).then(() => {
      return { data: data.ids };
    });
  };
  return { delete: _delete, deleteMany };
};
